<?php

require_once(dirname(__DIR__, 3).'/config/config.inc.php');

function copyImg($id_entity, $id_image = null, $url, $entity = 'products', $regenerate = true)
{
    $tempfile = tempnam(_PS_TMP_IMG_DIR_, 'ps_import');
    $watermark_types = explode(', ', Configuration::get('WATERMARK_TYPES'));

    switch($entity)
    {
        default:
        case 'products':
            $image_obj = new Image($id_image);
            $path = $image_obj->getPathForCreation();
        break;
        case 'categories':
            $path = _PS_CAT_IMG_DIR_.(int)$id_entity;
        break;
        case 'manufacturers':
            $path = _PS_MANU_IMG_DIR_.(int)$id_entity;
        break;
        case 'suppliers':
            $path = _PS_SUPP_IMG_DIR_.(int)$id_entity;
        break;
    }
    $url = str_replace(' ', '%20', trim($url));
    // Evaluate the memory required to resize the image: if it's too much, you can't resize it.
    if (!ImageManager::checkImageMemoryLimit($url))
    {
        return false;
    }

    if (Tools::copy($url, $tempfile))
    {
        ImageManager::resize($tempfile, $path.'.jpg');
        $image_types = ImageType::getImagesTypes($entity);
        if ($regenerate)
        {
            foreach($image_types as $image_type)
            {
                ImageManager::resize($tempfile, $path.'-'.stripslashes($image_type['name']).'.jpg', $image_type['width'], $image_type['height']);
                if (in_array($image_type['id_image_type'], $watermark_types))
                {
                    Hook::exec('actionWatermark', array('id_image'=> $id_image, 'id_product'=> $id_entity));
                }
            }
        }
    }
    else
    {
        unlink($tempfile);
        return false;
    }
    unlink($tempfile);
    return true;
}

function importProducts()
{
    $total = 0;
    
    if ((trim(Configuration::get('KEYINVOICE_API_URL')) == '') || (trim(Configuration::get('KEYINVOICE_API_KEY')) == ''))
    {
        echo json_encode(array('msg'=> 'É necessário configurar a chave e/ou o url da API do KeyInvoice.'));
        return;
    }

    $client = new SoapClient(trim(Configuration::get('KEYINVOICE_API_URL')), array('cache_wsdl' => WSDL_CACHE_DISK));
    $result = $client->authenticate(trim(Configuration::get('KEYINVOICE_API_KEY')));

    if (intval($result[0]) != 1)
	{
		echo json_encode(array('msg'=> utf8_decode($client->__soapCall('responseMessage', array($result[0], 0)))));
        return;
	}
	
    $APISession = $result[1];

    $result = $client->countProducts($APISession);
	$num_prod = intval($result[1]);
    $created_products = $updated_products = 0;
	
    set_time_limit(0);
    if ($num_prod > 0)
    {
        $aLangs = Language::getLanguages(false);
        
        for( $i = 0; $i <= $num_prod; $i += 25 )
        {
            $params = array('sid' => $APISession, 'offset' => $i);
			$result = $client->__soapCall('listProducts', $params);
 
            foreach($result->DAT as $kproduct)
            {
                $id_product = (int)Db::getInstance()->getValue('SELECT id_product FROM '._DB_PREFIX_.'product WHERE reference = \''.pSQL($kproduct->Ref).'\'');
         
                if (!$id_product)
                {
                    $product = new Product();
                    $product->reference = $kproduct->Ref;
                    $created_products++;
                }
                else 
                {
                    $product = new Product((int)$id_product);
                    $updated_products++;
                }

                Tools::displayAsDeprecated();
                $tax = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
                    SELECT t.`id_tax`
                    FROM `'._DB_PREFIX_.'tax` t
                    LEFT JOIN `'._DB_PREFIX_.'tax_lang` tl ON (t.id_tax = tl.id_tax)
                    WHERE t.`rate` = '.(float)($kproduct->TAX));
                $tax_id = $tax ? (int)($tax['id_tax']): false;

                if (!$tax_id)
                {
                    $tax = new Tax();
                    foreach($aLangs as $lang)
                    {
                        $tax->name[$lang['id_lang']] = utf8_decode('IVA'." ".floatval($kproduct->TAX).'%');
                    }
                    $tax->rate = $kproduct->TAX;
                    $tax->active = 1;
                    $tax->save();
                }
                else
                {
                    $tax = new Tax($tax_id);
                }

                $name_rule = 'KI_'.$kproduct->TAX.'%';
                $tax_rule_group = TaxRulesGroup::getIdByName($name_rule);
                if (!$tax_rule_group)
                {
                    $trg = new TaxRulesGroup();
                    $trg->name = $name_rule;
                    $trg->active = 1;
                    $trg->save();

                    TaxRule::deleteByGroupId($trg->id);
                    $countries = Country::getCountries((int)Configuration::get('PS_LANG_DEFAULT'), false, false, false);
                    foreach($countries as $country)
                    {
                        $tax_rule = new TaxRule();
                        $tax_rule->id_tax_rules_group = $trg->id;
                        $tax_rule->id_country = $country['id_country'];
                        $tax_rule->id_tax = $tax->id;
                        $tax_rule->save();
                    }
                }
                else
                {
                    $trg = new TaxRulesGroup($tax_rule_group);
                }

                $product->id_tax_rules_group = $trg->id;

                if (!$kproduct->TaxIncluded)
                {
                    $product->price = (float)$kproduct->Price;
                }
                else
                {
                    $product->price = number_format((float)floatval($kproduct->Price) / (1 + floatval($kproduct->TAX)/100), 9, '.', '');
                }
                $product->active = $kproduct->Active;

                if (trim($kproduct->EAN) !== '')
                {
                    if (Validate::isEan13($kproduct->EAN))
                    {
                        $product->ean13 = $kproduct->EAN;
                    }
                }

                foreach($aLangs as $lang)
                {
                    $product->name[$lang['id_lang']] = utf8_decode($kproduct->Name);
                    $product->description[$lang['id_lang']] = utf8_decode($kproduct->LongDescription);
                    $product->description_short[$lang['id_lang']] = utf8_decode($kproduct->ShortDescription);
                    $product->link_rewrite[$lang['id_lang']] = Tools::link_rewrite(utf8_decode($kproduct->Name));

                    $product->meta_description[$lang['id_lang']] = htmlspecialchars(utf8_decode($kproduct->ShortDescription));
                    $product->meta_title[$lang['id_lang']] = htmlspecialchars(utf8_decode($kproduct->Name));
                }
                $product->save();

                if ($kproduct->HasStocks)
                {
                    StockAvailable::setQuantity($product->id, 0, $kproduct->Stock);
                }

                if ($kproduct->Image !== '')
                {
                    $image = new Image();
                    //ver se imagem existe?
                    if (!$image->hasImages((int)Configuration::get('PS_LANG_DEFAULT'), $product->id))
                    {
                        $shops = Shop::getShops(true, null, true);
                        $image->id_product = $product->id;
                        $image->position = Image::getHighestPosition($product->id) + 1;
                        $image->cover = true;

                        if (($image->validateFields(false, true)) === true &&
						    ($image->validateFieldsLang(false, true)) === true && $image->add())
                        {
                            $image->associateTo($shops);

                            if (!copyImg($product->id, $image->id, $kproduct->Image, 'products', false))
                            {
                                $image->delete();
                            }
                        }
                    }
                }

                if (!isset($product->date_add) || empty($product->date_add))
                {
                    $product->date_add = date('Y-m-d H:i:s');
                }
                $product->date_upd = date('Y-m-d H:i:s');
                $product->save();


                if (intval($kproduct->IdFamily))
                {
                    $id_category = (int)Db::getInstance()->getValue('SELECT c.id_category 
                        FROM `'._DB_PREFIX_.'category` c
                        '.Shop::addSqlAssociation('category', 'c').'
                        WHERE keyinvoice_ref = \''.pSQL(utf8_decode($kproduct->IdFamily)).'\'  
                        GROUP BY c.id_category ');

                    if (intval($id_category))
                    {
                        $category = new Category((int)$id_category);
                        if (!Category::categoryExists($category->id_parent))
                        {
                            $category->id_parent = Category::getRootCategory()->id;
                        }
                    }
                    else
                    {
                        $category = new Category();
                        $category->id_parent = Category::getRootCategory()->id;
                    }

                    foreach($aLangs as $lang)
                    {
                        $category->name[$lang['id_lang']] = utf8_decode($kproduct->Family);
                        $category->description[$lang['id_lang']] = utf8_decode($kproduct->Family);
                        $category->link_rewrite[$lang['id_lang']] = Tools::link_rewrite(utf8_decode($kproduct->Family));
                    }
                    $category->save();

                    Db::getInstance()->execute("UPDATE "._DB_PREFIX_."category SET keyinvoice_ref = '".$kproduct->IdFamily ."' WHERE id_category = '".$category->id."'");

                    $product->addToCategories($category->id);
                    $product->id_category_default = $category->id;
                    
                    if(is_array($kproduct->FamilyTree) && !empty($kproduct->FamilyTree))
                    {
                        $parent = 0;
                        foreach($kproduct->FamilyTree as $family)
                        {
                            $id_category = (int)Db::getInstance()->getValue('SELECT c.id_category 
								FROM `'._DB_PREFIX_.'category` c
								'.Shop::addSqlAssociation('category', 'c').'
								WHERE keyinvoice_ref = \''.pSQL(utf8_decode($family->IdFamily)).'\'  
								GROUP BY c.id_category ');

                            if (intval($id_category))
                            {
                                $cat = new Category((int)$id_category);
                                if (!Category::categoryExists($cat->id_parent))
                                {
                                    if (intval($parent))
                                    {
                                        $cat->id_parent = $parent;
                                    }
                                    else
                                    {
                                        $cat->id_parent = Category::getRootCategory()->id;
                                    }
                                }
                            }
                            else
                            {
                                $cat = new Category();
                                if (intval($parent))
                                {
                                    $cat->id_parent = $parent;
                                }
                                else
                                {
                                    $cat->id_parent = Category::getRootCategory()->id;
                                }
                            }
                
                            foreach ($aLangs as $lang)
                            {
                                $cat->name[$lang['id_lang']] = utf8_decode($family->FamilyName);
								$cat->description[$lang['id_lang']] = utf8_decode($family->FamilyName);
								$cat->link_rewrite[$lang['id_lang']] = Tools::link_rewrite(utf8_decode($family->FamilyName));
                            }
                            $cat->save();
                
                            Db::getInstance()->execute("UPDATE "._DB_PREFIX_."category SET keyinvoice_ref = {$family->IdFamily} WHERE id_category = '".$cat->id."'");
					
							$product->addToCategories($cat->id);
							$product->id_category_default = $category->id;
							
							$parent = $cat->id;
                        }

                        if (intval($parent))
                        {
                            $category->id_parent = $parent;
                            $category->update();
                        }
                    }
                    $product->save();
                }
                // MARCAS
                if (intval($kproduct->IdBrand))
                {
                    $result = Db::getInstance()->getRow('SELECT `id_manufacturer`
					    FROM `'._DB_PREFIX_.'manufacturer`
					    WHERE `keyinvoice_ref` = \''.pSQL($kproduct->Brand->IdBrand).'\'');

                    if (isset($result['id_manufacturer']))
                    {
                        $brand = new Manufacturer((int)$result['id_manufacturer']);
                    }
                    else
                    {
                        $brand = new Manufacturer();
                    }

                    $brand->name = $kproduct->Brand->BrandName;
                    $brand->active = 1;
                    $brand->save();

                    Db::getInstance()->execute("UPDATE "._DB_PREFIX_."manufacturer SET keyinvoice_ref = {$kproduct->IdBrand} WHERE id_manufacturer = '".$brand->id."'");
                    
                    $product->id_manufacturer = $brand->id;
					$product->save();
                }
            }
        }
    }
    else
    {
        echo json_encode(array('msg' => 'Não foram encontrados produtos para importação.'));
        return;
    }
    $total = (int)$updated_products + (int)$created_products;
    echo json_encode(array('msg' => 'Importaçao Finalizada com sucesso! '.$total.' Artigo(s) sincronizado(s)'));
    return;
}

$req = $_POST;
$action = $req['action'];

switch ($action) 
{
    case 'import_products': {
        echo importProducts();
        break;
    }

}

